# Memory Benchmark

The `MemoryBenchmark.ino` was compiled with each `FEATURE_*` and the flash
memory and static RAM sizes were recorded. The `FEATURE_BASELINE` selection is
the baseline, and its memory usage  numbers are subtracted from the subsequent
`FEATURE_*` memory usage.

**Version**: AceTimeClock v1.0.4

**DO NOT EDIT**: This file was auto-generated using `make README.md`.

## How to Regenerate

To regenerate this README.md:

```
$ make clean_benchmarks
$ make benchmarks
$ make README.md
```

The `make benchmarks` target uses `collect.sh` script which calls `auniter.sh`
(https://github.com/bxparks/AUniter) to invoke the Arduino IDE programmatically.
It produces a `*.txt` file with the flash and ram usage information (e.g.
`nano.txt`). It now takes about 16 minutes to generate the `*.txt` files on my
quad-core Intel Core i7-3840QM CPU @ 2.80GHz laptop.

The `make README.md` command calls the `generated_readme.py` Python script which
generates this `README.md` file. The ASCII tables below are generated by the
`generate_table.awk` script, which takes each `*.txt` file and converts it to an
ASCII table.

## Library Size Changes

In v1.0.0:
* Initial split from AceTime v1.7.5 for v1.8.0.
* Convert `DS3231.h` into a template class for `<AceWire.h>`, replacing direct
  dependency on `<Wire.h>`.
    * Just including the `<Wire.h>` header causes flash memory to be consumed,
      even if `Wire` object is never used.
    * Saves 1000-1500 bytes of flash on AVR processors.
    * Saves 500 bytes of flash on SAMD.
    * Saves 4000 bytes of flash STM32.
    * Saves 500 bytes of flash on ESP8266.
    * Saves 3000-4000 bytes of flash on ESP32.
    * Saves 2500 bytes of flash on Teensy 3.2.
* Add benchmark for `DS3231Clock` separately from `SytemClock`.
* Add benchmarks for `SystemClockCoroutine` separately from `SystemClockLoop`.
* Rename `SystemClock` label to `SystemClockLoop`.

In v1.0.2:
* Fix bad merge in README.md.
* Create benchmarks for 3 variations of `DS3231Clock`: TwoWire, SimpleWire, and
  SimpleWireFast.
    * Make sure `<Wire.h>` is *not* included when using SimpleWireInterface and
      SimpleWireFastInterface.
    * Flash consumption reduced by 1600 bytes using SimpleWireInterface and
      2200 bytes using `SimpleWireInterface` on AVR.
    * Saves 800-8800 bytes of flash on 32-bit processors.

## Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* Arduino AVR Boards 1.8.3

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |    474/   11 |     0/    0 |
|----------------------------------------+--------------+-------------|
| DS3231Clock<TwoWire>                   |   4688/  249 |  4214/  238 |
| DS3231Clock<SimpleWire>                |   3030/   37 |  2556/   26 |
| DS3231Clock<SimpleWireFast>            |   2482/   37 |  2008/   26 |
|----------------------------------------+--------------+-------------|
| SystemClockLoop                        |   1500/   29 |  1026/   18 |
| SystemClockLoop+1 Basic zone           |   7028/  215 |  6554/  204 |
| SystemClockLoop+1 Extended zone        |  10038/  249 |  9564/  238 |
|----------------------------------------+--------------+-------------|
| SystemClockCoroutine                   |   2264/   41 |  1790/   30 |
| SystemClockCoroutine+1 Basic zone      |   7832/  227 |  7358/  216 |
| SystemClockCoroutine+1 Extended zone   |  10842/  261 | 10368/  250 |
+---------------------------------------------------------------------+

```

## Sparkfun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* SparkFun AVR Boards 1.1.13

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |   3470/  153 |     0/    0 |
|----------------------------------------+--------------+-------------|
| DS3231Clock<TwoWire>                   |   7674/  391 |  4204/  238 |
| DS3231Clock<SimpleWire>                |   6144/  179 |  2674/   26 |
| DS3231Clock<SimpleWireFast>            |   5478/  179 |  2008/   26 |
|----------------------------------------+--------------+-------------|
| SystemClockLoop                        |   4496/  171 |  1026/   18 |
| SystemClockLoop+1 Basic zone           |  10022/  355 |  6552/  202 |
| SystemClockLoop+1 Extended zone        |  13032/  389 |  9562/  236 |
|----------------------------------------+--------------+-------------|
| SystemClockCoroutine                   |   5260/  183 |  1790/   30 |
| SystemClockCoroutine+1 Basic zone      |  10826/  367 |  7356/  214 |
| SystemClockCoroutine+1 Extended zone   |  13836/  401 | 10366/  248 |
+---------------------------------------------------------------------+

```

## SAMD21 M0 Mini

* 48 MHz ARM Cortex-M0+
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* Sparkfun SAMD Boards 1.8.4

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  10064/    0 |     0/    0 |
|----------------------------------------+--------------+-------------|
| DS3231Clock<TwoWire>                   |  12688/    0 |  2624/    0 |
| DS3231Clock<SimpleWire>                |  11872/    0 |  1808/    0 |
|----------------------------------------+--------------+-------------|
| SystemClockLoop                        |  10760/    0 |   696/    0 |
| SystemClockLoop+1 Basic zone           |  14916/    0 |  4852/    0 |
| SystemClockLoop+1 Extended zone        |  16908/    0 |  6844/    0 |
|----------------------------------------+--------------+-------------|
| SystemClockCoroutine                   |  11288/    0 |  1224/    0 |
| SystemClockCoroutine+1 Basic zone      |  15444/    0 |  5380/    0 |
| SystemClockCoroutine+1 Extended zone   |  17436/    0 |  7372/    0 |
+---------------------------------------------------------------------+

```

(SAMD compiler does not produce RAM usage numbers.)

## STM32 Blue Pill

* STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* STM32duino 2.0.0

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  21420/ 3536 |     0/    0 |
|----------------------------------------+--------------+-------------|
| DS3231Clock<TwoWire>                   |  28448/ 3768 |  7028/  232 |
| DS3231Clock<SimpleWire>                |  25048/ 3584 |  3628/   48 |
|----------------------------------------+--------------+-------------|
| SystemClockLoop                        |  27100/ 3672 |  5680/  136 |
| SystemClockLoop+1 Basic zone           |  31136/ 3672 |  9716/  136 |
| SystemClockLoop+1 Extended zone        |  32880/ 3672 | 11460/  136 |
|----------------------------------------+--------------+-------------|
| SystemClockCoroutine                   |  27568/ 3676 |  6148/  140 |
| SystemClockCoroutine+1 Basic zone      |  31608/ 3676 | 10188/  140 |
| SystemClockCoroutine+1 Extended zone   |  33356/ 3676 | 11936/  140 |
+---------------------------------------------------------------------+

```

An entry of `-1` indicates that the memory usage exceeded the maximum of the
microcontroller and the compiler did not generate the desired information.

## ESP8266

* NodeMCU 1.0, 80MHz ESP8266
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* ESP8266 Boards 3.0.2

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 260089/27892 |     0/    0 |
|----------------------------------------+--------------+-------------|
| DS3231Clock<TwoWire>                   | 266069/28452 |  5980/  560 |
| DS3231Clock<SimpleWire>                | 264305/27980 |  4216/   88 |
|----------------------------------------+--------------+-------------|
| SystemClockLoop                        | 262905/27992 |  2816/  100 |
| SystemClockLoop+1 Basic zone           | 268881/28552 |  8792/  660 |
| SystemClockLoop+1 Extended zone        | 271025/28696 | 10936/  804 |
|----------------------------------------+--------------+-------------|
| SystemClockCoroutine                   | 263449/27992 |  3360/  100 |
| SystemClockCoroutine+1 Basic zone      | 269425/28552 |  9336/  660 |
| SystemClockCoroutine+1 Extended zone   | 271585/28696 | 11496/  804 |
+---------------------------------------------------------------------+

```

## ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* ESP32 Boards 1.0.6

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 197748/13084 |     0/    0 |
|----------------------------------------+--------------+-------------|
| DS3231Clock<TwoWire>                   | 217554/14312 | 19806/ 1228 |
| DS3231Clock<SimpleWire>                | 208762/13584 | 11014/  500 |
|----------------------------------------+--------------+-------------|
| SystemClockLoop                        | 206546/13424 |  8798/  340 |
| SystemClockLoop+1 Basic zone           | 211470/13424 | 13722/  340 |
| SystemClockLoop+1 Extended zone        | 213490/13424 | 15742/  340 |
|----------------------------------------+--------------+-------------|
| SystemClockCoroutine                   | 207158/13432 |  9410/  348 |
| SystemClockCoroutine+1 Basic zone      | 212082/13432 | 14334/  348 |
| SystemClockCoroutine+1 Extended zone   | 214090/13432 | 16342/  348 |
+---------------------------------------------------------------------+

```

RAM usage remains constant as more objects are created, which indicates that an
initial pool of a certain minimum size is created regardless of the actual RAM
usage by objects.

## Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* Teensyduino 1.55

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  10184/ 4152 |     0/    0 |
|----------------------------------------+--------------+-------------|
| DS3231Clock<TwoWire>                   |  14416/ 4820 |  4232/  668 |
| DS3231Clock<SimpleWire>                |  12280/ 4152 |  2096/    0 |
|----------------------------------------+--------------+-------------|
| SystemClockLoop                        |  10392/ 4160 |   208/    8 |
| SystemClockLoop+1 Basic zone           |  19704/ 4160 |  9520/    8 |
| SystemClockLoop+1 Extended zone        |  23460/ 4160 | 13276/    8 |
|----------------------------------------+--------------+-------------|
| SystemClockCoroutine                   |  11600/ 4164 |  1416/   12 |
| SystemClockCoroutine+1 Basic zone      |  20968/ 4164 | 10784/   12 |
| SystemClockCoroutine+1 Extended zone   |  24724/ 4164 | 14540/   12 |
+---------------------------------------------------------------------+

```

